package com.szyd.uhfdemo;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ListView;
import android.widget.TextView;

import com.szyd.event.BackResult;
import com.szyd.event.GetRFIDThread;
import com.szyd.event.InventoryResult;
import com.szyd.util.DataAdapter;
import com.szyd.util.MUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LeftFragment extends Fragment implements View.OnClickListener, BackResult {

    private TextView read_RFID;
    private TextView tagNumbers, readNumbers, useTimes;

    private String tagNumber, readNumber, takeTime;

    private long startTime, usTim, pauseTime;
    private DataAdapter mListAdapter;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_left, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView(view);
    }

    private void initView(View v) {
        tagNumber = getString(R.string.tag_number);
        readNumber = getString(R.string.read_number);
        takeTime = getString(R.string.user_time);
        read_RFID = v.findViewById(R.id.read_RFID);
        v.findViewById(R.id.clear_Data).setOnClickListener(this); //clear data
        ListView specific_Msg = v.findViewById(R.id.specific_Msg);

        tagNumbers = v.findViewById(R.id.tagNumbers);
        readNumbers = v.findViewById(R.id.readNumbers);
        useTimes = v.findViewById(R.id.useTimes);

        read_RFID.setOnClickListener(this);
        read_RFID.setText(GetRFIDThread.getInstance().isIfPostMsg() ? R.string.stop_rfid : R.string.read_rfid);

        mListAdapter = new DataAdapter(getActivity(), realDataMap, realKeyList);
        specific_Msg.setAdapter(mListAdapter);
        GetRFIDThread.getInstance().setBackResult(this);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.read_RFID:
                startOrStopRFID();
                break;
            case R.id.clear_Data:  //clear data
                if (GetRFIDThread.getInstance().isIfPostMsg()) {
                    MUtil.show("Please stop the tag reading before clearing the data...");
                } else {
                    dataMap.clear();
                    realDataMap.clear();
                    realKeyList.clear();
                    rNumber = usTim = pauseTime = 0;
                    useTimes.setText(takeTime);
                    tagNumbers.setText(tagNumber);
                    readNumbers.setText(readNumber);
                    mListAdapter.notifyDataSetChanged();
                }
                break;
        }
    }

    //Start or stop the RFID Inventory
    public void startOrStopRFID() {
        boolean flag = !GetRFIDThread.getInstance().isIfPostMsg();
        if (flag) {
            MyApp.getMyApp().getIdataLib().startInventoryTag();
            long tmepTime = pauseTime;
            startTime = System.currentTimeMillis() - tmepTime;
        } else {
            MyApp.getMyApp().getIdataLib().stopInventory();
        }
        GetRFIDThread.getInstance().setIfPostMsg(flag);
        read_RFID.setText(flag ? R.string.stop_rfid : R.string.read_rfid);
    }

    private long rNumber = 0;

    private Map<String, Integer> dataMap = new HashMap<>(); //Data

    //Scanned EPC data
    @Override
    public void postResult(String[] tagData) {
        ++rNumber;
        String epc = tagData[1]; //Get EPC
        Integer number = dataMap.get(epc);//If the EPC exist, update read times
        if (number == null) {
            dataMap.put(epc, 1);
            updateUI(epc, 1);
        } else {
            int newNB = number + 1;
            dataMap.put(epc, newNB);
            updateUI(epc, newNB);
        }
    }

    @Override
    public void onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_F8) //press the pistol trigger,the key value default as 138
            startOrStopRFID();
    }

    private Map<String, Integer> realDataMap = new HashMap<>();
    private List<String> realKeyList = new ArrayList<>();

    private void updateUI(final String epc, final int readNumberss) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (readNumberss > 1) {
                    realDataMap.put(epc, readNumberss); //update the read times of tags
                } else {
                    realDataMap.put(epc, 1);
                    realKeyList.add(epc);
                }
                long endTime = System.currentTimeMillis();
                pauseTime = usTim = endTime - startTime;
                useTimes.setText(takeTime + usTim); //time consumed
                tagNumbers.setText(tagNumber + realDataMap.size());//tag quantity
                readNumbers.setText(readNumber + rNumber);
                mListAdapter.notifyDataSetChanged();
                if (MyApp.ifOpenSound)
                    MyApp.getMyApp().playSound();

            }
        });
    }


}
