package com.szyd.uhfdemo;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.SystemClock;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ImageView;

import com.szyd.event.GetRFIDThread;
import com.szyd.event.OnKeyDownListener;
import com.szyd.event.OnKeyListener;
import com.szyd.util.MLog;
import com.szyd.util.MUtil;

import java.util.Timer;
import java.util.TimerTask;

import realid.rfidlib.EmshConstant;

import static realid.rfidlib.EmshConstant.EmshBatteryPowerMode.EMSH_PWR_MODE_CHG_FULL;
import static realid.rfidlib.EmshConstant.EmshBatteryPowerMode.EMSH_PWR_MODE_CHG_GENERAL;
import static realid.rfidlib.EmshConstant.EmshBatteryPowerMode.EMSH_PWR_MODE_CHG_QUICK;
import static realid.rfidlib.EmshConstant.EmshBatteryPowerMode.EMSH_PWR_MODE_DSG_UHF;
import static realid.rfidlib.EmshConstant.EmshBatteryPowerMode.EMSH_PWR_MODE_STANDBY;

public class MainActivity extends AppCompatActivity {

    private LeftFragment mLeftFragment;
    private RightFragment mRightFragment;
    private SearchFragment mSearchFragment;
    private FragmentManager manager;
    private ImageView toLeft, toRight, searchTag;

    private Object currentFragment;
    private GetRFIDThread rfidThread = GetRFIDThread.getInstance();//Inventory Thread
    private Timer mTimer = null;
    private TimerTask mTimerTask = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        init();
    }

    private void init() {
        if (getSupportActionBar() != null)
            getSupportActionBar().hide();
        toLeft = findViewById(R.id.toLeft);
        toRight = findViewById(R.id.toRight);
        searchTag = findViewById(R.id.searchTag);
        manager = getSupportFragmentManager();
        setCurrentPage(0);
        MLog.e("poweron = " + MyApp.getMyApp().getIdataLib().powerOn());
        rfidThread.start();
        monitorEmsh();
    }

    //Getting Pistol Grip Power Status
    private void monitorEmsh() {
        mEmshStatusReceiver = new EmshStatusBroadcastReceiver();
        IntentFilter intentFilter = new IntentFilter(EmshConstant.Action.INTENT_EMSH_BROADCAST);
        registerReceiver(mEmshStatusReceiver, intentFilter);

        mTimer = new Timer();
        mTimerTask = new TimerTask() {
            @Override
            public void run() {
                Intent intent = new Intent(EmshConstant.Action.INTENT_EMSH_REQUEST);
                intent.putExtra(EmshConstant.IntentExtra.EXTRA_COMMAND, EmshConstant.Command.CMD_REFRESH_EMSH_STATUS);
                sendBroadcast(intent);
            }
        };
        mTimer.schedule(mTimerTask, 0, 1000);
    }

    //Main Fragment
    private void setCurrentPage(int tabPage) {
        FragmentTransaction transaction = manager.beginTransaction();
        switch (tabPage) {
            case 0: //Home interface
                currentFragment = mLeftFragment = (mLeftFragment == null ? new LeftFragment() : mLeftFragment);
                break;
            case 1: //To set the interface
                currentFragment = mRightFragment = (mRightFragment == null ? new RightFragment() : mRightFragment);
                break;
            case 2://Tag searching interface
                currentFragment = mSearchFragment = (mSearchFragment == null ? new SearchFragment() : mSearchFragment);
                break;
        }
        transaction.replace(R.id.showData, (Fragment) currentFragment);
        transaction.commit();
    }

    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.toLeft:
                setCurrentPage(0);
                toLeft.setImageResource(R.drawable.main_click);
                toRight.setImageResource(R.drawable.set_noclick);
                searchTag.setImageResource(R.drawable.search_noclcik);
                break;
            case R.id.toRight:
                if (!GetRFIDThread.getInstance().isIfPostMsg()) {
                    setCurrentPage(1);
                    toRight.setImageResource(R.drawable.set_click);
                    toLeft.setImageResource(R.drawable.main_noclick);
                    searchTag.setImageResource(R.drawable.search_noclcik);
                } else {
                    MUtil.show("Please stop tag reading firstly.");
                }
                break;
            case R.id.searchTag:
                if (!GetRFIDThread.getInstance().isIfPostMsg()) {
                    setCurrentPage(2);
                    toRight.setImageResource(R.drawable.set_noclick);
                    toLeft.setImageResource(R.drawable.main_noclick);
                    searchTag.setImageResource(R.drawable.search_clcik);
                } else {
                    MUtil.show("Please stop tag reading firstly.");
                }
                break;
        }
    }

    //Start or stop the inventory thread by trigger
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        MLog.e(" press keycode  = " + keyCode);
        if (currentFragment instanceof OnKeyDownListener)
            ((OnKeyDownListener) currentFragment).onKeyDown(keyCode, event);
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (currentFragment instanceof OnKeyListener)
            ((OnKeyListener) currentFragment).onKeyUp(keyCode, event);
        return super.onKeyUp(keyCode, event);
    }

    @Override
    protected void onPause() {
        super.onPause();
        recyleResoure();
    }

    private volatile long lastTime = 0;

    @Override
    public void onBackPressed() {
        long currentTime = SystemClock.currentThreadTimeMillis();
        if (lastTime != 0 && currentTime - lastTime < 500) {
            recyleResoure();
        } else {
            MUtil.show("Double click to exit the App");
        }
        lastTime = currentTime;
    }

    //RFID power off, recycle the thread and exit
    private void recyleResoure() {
        if (mEmshStatusReceiver != null) {
            unregisterReceiver(mEmshStatusReceiver);
            mEmshStatusReceiver = null;
        }
        if (mTimer != null || mTimerTask != null) {
            mTimerTask.cancel();
            mTimer.cancel();
            mTimerTask = null;
            mTimer = null;
        }
        rfidThread.destoryThread();
        MLog.e("powoff = " + MyApp.getMyApp().getIdataLib().powerOff());
        System.exit(0);
    }

    private EmshStatusBroadcastReceiver mEmshStatusReceiver;

    private int oldStatue = -1;

    public class EmshStatusBroadcastReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {

            if (EmshConstant.Action.INTENT_EMSH_BROADCAST.equalsIgnoreCase(intent.getAction())) {

                int sessionStatus = intent.getIntExtra("SessionStatus", 0);
                int batteryPowerMode = intent.getIntExtra("BatteryPowerMode", -1);
                //  MLog.e("sessionStatus = " + sessionStatus + "  batteryPowerMode  = " + batteryPowerMode);
                if ((sessionStatus & EmshConstant.EmshSessionStatus.EMSH_STATUS_POWER_STATUS) != 0) {
                    // Current status of pistol grip power
                    if (batteryPowerMode == oldStatue) {
                        return;
                    }
                    oldStatue = batteryPowerMode;
                    switch (batteryPowerMode) {
                        case EMSH_PWR_MODE_STANDBY:
                            MUtil.cancelWaringDialog();
                            MyApp.getMyApp().getIdataLib().powerOn();
                            break;
                        case EMSH_PWR_MODE_DSG_UHF:
                            MUtil.show("RFID Power On...");
                            break;
                        case EMSH_PWR_MODE_CHG_GENERAL:
                        case EMSH_PWR_MODE_CHG_QUICK:
                            MUtil.show("Charging...");
                            break;
                        case EMSH_PWR_MODE_CHG_FULL:
                            MUtil.show("Pistol Battery Full");
                            break;
                    }
                } else {
                    MUtil.warningDialog(MainActivity.this);
                }
            }
        }
    }

}
